from typing import Sequence
from .assetpairproperties import AssetPairProperties
from .marketdata import MarketData
from .placeorderrequest import PlaceOrderRequest
from .placeorderresponse import PlaceOrderResponse
from .cancelorderrequest import CancelOrderRequest
from .cancelorderresponse import CancelOrderResponse
from .statusresponse import StatusResponse


class Exchange(object):
    def __init__(self):
        pass

    def get_time(self) -> int:
        raise NotImplementedError("This method should be overwritten.")

    def get_market(self) -> MarketData:
        raise NotImplementedError("This method should be overwritten.")

    def get_assets(self) -> Sequence[AssetPairProperties]:
        raise NotImplementedError("This method should be overwritten.")

    def place_order(self, request: PlaceOrderRequest) -> PlaceOrderResponse:
        raise NotImplementedError("This method should be overwritten.")

    def cancel_order(self, request: CancelOrderRequest) -> CancelOrderResponse:
        raise NotImplementedError("This method should be overwritten.")

    def status(self) -> StatusResponse:
        raise NotImplementedError("This method should be overwritten.")



# "public interface"
from .account import Account
from .assetpair import AssetPair
from .assetpairproperties import AssetPairProperties
from .cancelorderrequest import CancelOrderRequest
from .cancelorderresponse import CancelOrderResponse
from .exchange import Exchange
from .exchangefactory import ExchangeFactory
from .marketdata import MarketData
from .order import Order
from .placeorderrequest import PlaceOrderRequest
from .placeorderresponse import PlaceOrderResponse
from .statusresponse import StatusResponse
from .ticker import Ticker

__all__ = ['Account', 'AssetPair', 'AssetPairProperties', 'CancelOrderRequest, CancelOrderResponse', 'Exchange', 'ExchangeFactory', 'MarketData', 'Order', 'PlaceOrderRequest', 'PlaceOrderResponse', 'StatusResponse', 'Ticker']



class CancelOrderResponse(object):
    def __init__(self, success: bool, error: str):
        self.__success = success
        self.__error = error

    def success(self) -> bool:
        return self.__success

    def error(self) -> str:
        return self.__error

    @staticmethod
    def from_json(data):
        return CancelOrderResponse(data['success'], data['error'])


import json
from . import AssetPair


class Ticker(object):
    def __init__(self, exchange: str, pair: AssetPair, timestamp: int, bid: float, bid_volume: float,
                 ask: float, ask_volume: float):
        self.__exchange = exchange
        self.__pair = pair
        self.__timestamp = timestamp
        self.__bid = bid
        self.__bid_volume = bid_volume
        self.__ask = ask
        self.__ask_volume = ask_volume

    def exchange(self) -> str:
        return self.__exchange

    def pair(self) -> AssetPair:
        return self.__pair

    def timestamp(self) -> int:
        return self.__timestamp

    def bid(self) -> float:
        return self.__bid

    def bid_volume(self) -> float:
        return self.__bid_volume

    def ask(self) -> float:
        return self.__ask

    def ask_volume(self) -> float:
        return self.__ask_volume

    def mid(self) -> float:
        return (self.bid() + self.ask())/2

    def spread(self) -> float:
        return self.ask()-self.bid()

    @staticmethod
    def from_json(data):
        return Ticker(data['exchange'], AssetPair.from_json(data['pair']), data['timestamp'], data['bid'],
                      data['bid_volume'], data['ask'], data['ask_volume'])

    def __str__(self):
        return json.dumps(self, default=lambda o: o.__dict__)



from typing import Sequence
from marketmaker import version
from .exchange import Exchange
from .marketdata import MarketData
from .assetpairproperties import AssetPairProperties
from .placeorderrequest import PlaceOrderRequest
from .placeorderresponse import PlaceOrderResponse
from .cancelorderrequest import CancelOrderRequest
from .cancelorderresponse import CancelOrderResponse
from .statusresponse import StatusResponse
import requests


class ExchangeProxy(Exchange):
    def __init__(self, base_url):
        self._base_url = base_url
        self._apiversion = '0'
        self._session = requests.Session()
        self._session.headers.update({
            'User-Agent': 'vasilv/' + version.__version__ + ' (+' + version.__url__ + ')'
        })

    def get_time(self) -> int:
        response_json = self._query('/time', 'GET');
        return int(response_json)

    def get_market(self) -> MarketData:
        response_json = self._query('/market', 'GET')
        return MarketData.from_json(response_json)

    def get_assets(self) -> Sequence[AssetPairProperties]:
        response_json = self._query('/assets', 'GET')
        return [AssetPairProperties.from_json(data) for data in response_json]

    def place_order(self, request: PlaceOrderRequest) -> PlaceOrderResponse:
        response_json = self._query('/open', 'POST', data=request.to_json())
        return PlaceOrderResponse.from_json(response_json)

    def cancel_order(self, request: CancelOrderRequest) -> CancelOrderResponse:
        response_json = self._query('/cancel', 'POST', data=request.to_json())
        return CancelOrderResponse.from_json(response_json)

    def status(self) -> StatusResponse:
        response_json = self._query('/status', 'GET')
        return StatusResponse.from_json(response_json)

    def _query(self, urlpath, method, data=None, headers=None, timeout=None, auth=None):
        """ Low-level query handling.

        .. note::
           Use :py:meth:`query_private` or :py:meth:`query_public`
           unless you have a good reason not to.

        :param urlpath: API URL path sans host
        :type urlpath: str
        :param urlpath: HTTP method type - one of GET or POST
        :type urlpath: str
        :param data: API request parameters
        :type data: dict
        :param headers: (optional) HTTPS headers
        :type headers: dict
        :param timeout: (optional) if not ``None``, a :py:exc:`requests.HTTPError`
                        will be thrown after ``timeout`` seconds if a response
                        has not been received
        :type timeout: int or float
        :returns: :py:meth:`requests.Response.json`-deserialised Python object
        :raises: :py:exc:`requests.HTTPError`: if response status not successful

        """
        if data is None:
            data = {}
        if headers is None:
            headers = {}

        headers['Content-Type'] = 'application/json'

        url = self._base_url + urlpath

        if method == 'GET':
            response = self._session.get(url, json=data, headers=headers, timeout=timeout, auth=auth)
        elif method == 'POST':
            response = self._session.post(url, json=data, headers=headers, timeout=timeout, auth=auth)
        elif method == 'DELETE':
            response = self._session.delete(url, json=data, headers=headers, timeout=timeout, auth=auth)
        else:
            raise Exception("Method not supported.")

        if response.status_code not in (200, 201, 202):
            response.raise_for_status()

        return response.json()



import json
from .assetpair import AssetPair


class PlaceOrderRequest(object):
    def __init__(self, pair: AssetPair, price: float, volume: float, side: str):
        self.__pair = pair
        self.__price = price
        self.__volume = volume
        self.__side = side

    def pair(self) -> AssetPair:
        return self.__pair

    def price(self) -> float:
        return self.__price

    def volume(self) -> float:
        return self.__volume

    def side(self) -> str:
        return self.__side

    def to_json(self):
        return {
            'pair': self.pair().to_json(),
            'price': self.price(),
            'volume': self.volume(),
            'side': self.side()
        }

    def __str__(self):
        return json.dumps(self, default=lambda o: o.__dict__)

    def __eq__(self, other):
        """Overrides the default implementation"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return NotImplemented

    def __hash__(self):
        """Overrides the default implementation"""
        return hash(tuple(sorted(self.__dict__.items())))



import json
from typing import Sequence
from .account import Account
from .ticker import Ticker


class MarketData(object):
    def __init__(self, account: Account, market: Sequence[Ticker]):
        self.__account = account
        self.__market = market

    def account(self):
        return self.__account

    def market(self):
        return self.__market

    @staticmethod
    def from_json(data):
        return MarketData(Account.from_json(data['account']),
                          [Ticker.from_json(ticker_json) for ticker_json in data['market']])



class PlaceOrderResponse(object):
    def __init__(self, success: bool, order_id: str):
        self.__success = success
        self.__order_id = order_id

    def success(self) -> bool:
        return self.__success

    def order_id(self) -> str:
        return self.__order_id

    @staticmethod
    def from_json(data):
        return PlaceOrderResponse(data['success'], data['order_id'])



import json
from typing import Sequence, Mapping
from .order import Order


class Account(object):
    def __init__(self, sequence_number: int, orders: Sequence[Order], balance: Mapping[str, float]):
        self.__sequence_number = sequence_number
        self.__orders = orders
        self.__balance = balance

    def sequence_number(self) -> int:
        return self.__sequence_number

    def orders(self) -> Sequence[Order]:
        return self.__orders

    def balance(self) -> Mapping[str, float]:
        return self.__balance

    @staticmethod
    def from_json(data):
        return Account(int(data['sequence_number']),
                       [Order.from_json(order_data) for order_data in data['orders']],
                       dict([(k, v) for k, v in data['balance'].items()]))

    def __str__(self):
        return json.dumps(self, default=lambda o: o.__dict__)

    def __eq__(self, other):
        """Overrides the default implementation"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return NotImplemented

    def __hash__(self):
        """Overrides the default implementation"""
        return hash(tuple(sorted(self.__dict__.items())))



import logging
import time
from typing import Mapping
from .exchange import Exchange
from .exchangeproxy import ExchangeProxy


class ExchangeFactory(object):
    def __init__(self):
        pass

    @staticmethod
    def instance(config: Mapping[str, str]) -> Exchange:
        logger = logging.getLogger('ExchangeFactory')

        if config['type'] == 'exchange-proxy':
            exchange = ExchangeProxy(config['base_url'])
            while True:
                try:
                    if exchange.status().healthy():
                        return exchange
                except Exception as ex:
                    pass

                logger.warning("Exchange not ready. Sleeping for 10s...")
                time.sleep(10)



import json
from .assetpair import AssetPair


class AssetPairProperties(object):
    def __init__(self, pair: AssetPair, exchange_specific_id: str, price_significant_digits: int,
                 size_significant_digits: int, max_size: float):
        self.__pair = pair
        self.__exchange_specific_id = exchange_specific_id
        self.__price_significant_digits = price_significant_digits
        self.__size_significant_digits = size_significant_digits
        self.__max_size = max_size

    def pair(self) -> AssetPair:
        return self.__pair

    def exchange_specific_id(self) -> str:
        return self.__exchange_specific_id

    def price_significant_digits(self) -> int:
        return self.__price_significant_digits

    def size_significant_digits(self) -> int:
        return self.__size_significant_digits

    def max_size(self) -> float:
        return self.__max_size

    @staticmethod
    def from_json(data):
        return AssetPairProperties(AssetPair.from_json(data['pair']), data['exchange_specific_id'],
                                   data['price_significant_digits'], data['size_significant_digits'], data['max_size'])

    def __str__(self):
        return json.dumps(self, default=lambda o: o.__dict__)



class StatusResponse(object):
    def __init__(self, healthy: bool, error: str):
        self.__healthy = healthy
        self.__error = error

    def healthy(self) -> bool:
        return self.__healthy

    def error(self) -> str:
        return self.__error

    @staticmethod
    def from_json(data):
        return StatusResponse(data['healthy'], data['error'])



class CancelOrderRequest(object):
    def __init__(self, order_id: str):
        self.__order_id = order_id

    def order_id(self) -> str:
        return self.__order_id

    def to_json(self):
        return {
            'order_id': self.order_id()
        }



from distutils.core import setup
setup(
  name = 'onemarket',
  packages = ['onemarket'],
  url='https://bitbucket.org/onemarkettech/onemarket-api/',
  download_url='https://bitbucket.org/onemarkettech/onemarket-api/get/0.5.tar.gz',
  version = '0.5',
  description = 'An API for the Onemarket platform',
  author='Vasil Vasilev',
  author_email='vasil@onemarket.tech',
  license='Apache 2.0',
  zip_safe=False
)



class AssetPair(object):
    def __init__(self, base, quote):
        self.__base = base
        self.__quote = quote

    def base(self):
        return self.__base

    def quote(self):
        return self.__quote

    @staticmethod
    def from_json(data):
        return AssetPair(data['base'], data['quote'])

    def to_json(self):
        return {
            'base': self.base(),
            'quote': self.quote()
        }

    def __str__(self):
        return "%s%s" % (self.__base, self.__quote)

    def __hash__(self):
        return hash((self.__base, self.__quote))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and other.__base == self.__base and other.__quote == self.__quote



from .assetpair import AssetPair


class Order(object):
    def __init__(self, exchange: str, id: str, pair: AssetPair, side: str, price: float,
                 volume: float, expire_at: int, state: str):
        self.__exchange = exchange
        self.__id = id
        self.__pair = pair
        self.__side = side
        self.__price = price
        self.__volume = volume
        self.__expire_at = expire_at
        self.__state = state

    def exchange(self) -> str:
        return self.__exchange

    def id(self) -> str:
        return self.__id

    def pair(self) -> AssetPair:
        return self.__pair

    def side(self) -> str:
        return self.__side

    def price(self) -> float:
        return self.__price

    def volume(self) -> float:
        return self.__volume

    def expire_at(self) -> int:
        return self.__expire_at

    def state(self):
        return self.__state

    @staticmethod
    def from_json(data):
        return Order(data['exchange'], data['id'], AssetPair.from_json(data['pair']), data['side'],
                     data['price'], data['volume'], None, data['state'])
